import os,os.path,string
import cherrypy
from Common import config,DetConfig
import pluginCondDBPyInterface as CondDB
from CondCore.Utilities import iovInspector as inspect
import simplejson as json
from cherrypy.lib import static
class IOV(object):
    def __init__(self,configfilepath):
        self.__config=config.Config()
        self.__config.from_ini(configfilepath)
        self.__authpath=self.__config.common.auth_path
        self.__datasavedir=self.__config.common.datasave_dir
        self.__detconfig=DetConfig.DetConfig(self.__config.common.detector_conf)
        self.__cmsframe=CondDB.FWIncantation()
        #self.__logdb=str(self.__config.iov.log_conn)
        self.__rdbms=CondDB.RDBMS(self.__authpath)
    @cherrypy.expose
    def index(self):
        """Top level page
        """
        overviewstr=['IOV inspection service']
        overviewstr.append(self.__authpath)
        overviewstr.append(self.__datasavedir)
        return (' ').join(overviewstr)   
    
    def __getconnectstr(self,det,service,schema,viacache):
        """Assemble full connect string with given parameters
        """
        connect_str=''
        if not viacache:
            if (service.find(":"))==-1 :
               connect_str+='oracle://'
            connect_str+=service+'/'+schema          
        else:            
            #knownservices=self.__detconfig.services(det)
            #if '/'.join([service,schema]) not in knownservices:
            #   return ''
            servletname=self.__detconfig.serviceToServlet(det,service)
            connect_str+='frontier://(serverurl=http://cmsfrontier.cern.ch:8000/'+servletname+')/'+schema
        if len(connect_str)==0:
            return "Application Server Error: requested service "+service+" "+schema+"is not configured"
        return connect_str
    
    @cherrypy.expose
    def getTags(self,det,service,schema,viacache=0):
        """Get list of tags
        """
        result={"status":"",
                "data":""
                }
        connect_str=self.__getconnectstr(det,service,schema,viacache)
        if len(connect_str)==0:
            result.status="Application Server Error: requested service "+service+" "+schema+"is not configured"
        try:
            db = self.__rdbms.getDB(connect_str)
            tags = db.allTags()
            result["status"]="OK"
            result["data"]=tags
        except Exception, er:
            result["status"]=str(er)
        except RuntimeError:
            result["status"]="RuntimeError"
        except:
            result["status"]="Unknown Error"
        return json.dumps(result)
    
    @cherrypy.expose
    def getPayloadSummariesForTag(self,det,service,schema,tag,since=0,till=0,head=0,tail=0,viacache=0):
        """Get payload summary info for given tag in the given iov range
        """
        result={"status":"",
                "data":[]
               }
        connect_str=self.__getconnectstr(det,service,schema,viacache)
        try:
            db = self.__rdbms.getDB(connect_str)
            result["data"]=inspect.Iov(db,tag).payloadSummaries()
            result["status"]="OK"
        except Exception, er:
            result["status"]=str(er)
        except RuntimeError:
            result["status"]="RuntimeError"
        except:
            result["status"]="Unknown Error"
        return json.dumps(result)
    
    @cherrypy.expose
    def getIOVForTag(self,det,service,schema,tag,since=0,till=0,head=0,tail=0,viacache=0):
        """Get detailed iov info for given tag in the given range
        """
        result={"status":"",
                "comment":"",
                "revision":"",
                "payloadContainerName":"",
                "timetype":"",
                "data":[]
                }
        connect_str=self.__getconnectstr(det,service,schema,viacache)
        if len(connect_str)==0:
            result["status"]= "Application Server Error: requested service "+service+" "+schema+"is not configured"
            return json.dumps(result)
        try:
            db = self.__rdbms.getDB(connect_str)
            v=inspect.Iov(db,tag)
            result['comment']=v.comment()
            result['revision']=v.revision()
            result['payloadContainerName']=v.payloadContainerName()
            result['timetype']=str(v.timetype())            
            suma=v.list()
            for s in suma:
              r={}
              r["payloadToken"]=s[0]
              r["since"]=s[1]
              r["till"]=s[2]
              result["data"].append(r)
            result["status"]="OK"
        except Exception, er:
            print 'PyCoral Error ',str(er)
            result["status"]=str(er)
        except RuntimeError:
            result["status"]="RuntimeError"
        except:
            result["status"]="Unknown Error"
#        print json.dumps(result)
        return json.dumps(result)

    @cherrypy.expose
    def getWhatForTag(self,det,service,schema,tag,viacache):
        """get What() information defined by the payload
        """
        result={"status":"","data":""}
        connect_str=self.__getconnectstr(det,service,schema,viacache)
        if len(connect_str)==0:
           result["status"]= "Application Server Error: requested service "+service+" "+schema+"is not configured"
           return json.dumps(result)
        try:
            db = self.__rdbms.getDB(connect_str)
            result["data"]= inspect.extractorWhat(db,tag)
        except Exception, er:
            print 'PyCoral Error ',str(er)
            result["status"]=str(er)
        except RuntimeError:
            result["status"]="RuntimeError"
        except:
            result["status"]="Unknown Error"                
        return json.dumps(result)
 

    @cherrypy.expose
    def getTrendForTag(self,det,service,schema,tag,since=0,till=0,extractor=0,viacache=0):
        """get trend data with the given trend extractor for a given tag in the given range. This is an example. Detector should parse or treat its trend data accordingly in the detector specific modules
        """
        result={"status":"",
                "data":""
               }
        connect_str=self.__getconnectstr(det,service,schema,viacache)
        e={}
        if extractor != 0:
          try:
            e=json.loads(extractor)
          except ValueError,er:
            result["status"]="JSON error "+er
            return result
        if len(connect_str)==0:
            result["status"]= "Application Server Error: requested service "+service+" "+schema+"is not configured"
            return json.dumps(result)
        try:
            db = self.__rdbms.getDB(connect_str)
            trenddata=inspect.Iov(db,tag).trend(e)
            result["data"]=trenddata            
            result["status"]="OK"
        except Exception, er:
            print 'PyCoral Error ',str(er)
            result["status"]=str(er)
        except RuntimeError:
            result["status"]="RuntimeError"
        except:
            result["status"]="Unknown Error"                
        return json.dumps(result)
        
